""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/sdn_connector
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SdnConnectorExternalaccountlistRegionlistItem(TypedDict, total=False):
    """Nested item for external-account-list.region-list field."""
    region: str


class SdnConnectorNicIpItem(TypedDict, total=False):
    """Nested item for nic.ip field."""
    name: str
    private_ip: str
    public_ip: str
    resource_group: str


class SdnConnectorRoutetableRouteItem(TypedDict, total=False):
    """Nested item for route-table.route field."""
    name: str
    next_hop: str


class SdnConnectorGcpprojectlistGcpzonelistItem(TypedDict, total=False):
    """Nested item for gcp-project-list.gcp-zone-list field."""
    name: str


class SdnConnectorServerlistItem(TypedDict, total=False):
    """Nested item for server-list field."""
    ip: str


class SdnConnectorExternalaccountlistItem(TypedDict, total=False):
    """Nested item for external-account-list field."""
    role_arn: str
    external_id: str
    region_list: str | list[str] | list[SdnConnectorExternalaccountlistRegionlistItem]


class SdnConnectorNicItem(TypedDict, total=False):
    """Nested item for nic field."""
    name: str
    peer_nic: str
    ip: str | list[str] | list[SdnConnectorNicIpItem]


class SdnConnectorRoutetableItem(TypedDict, total=False):
    """Nested item for route-table field."""
    name: str
    subscription_id: str
    resource_group: str
    route: str | list[str] | list[SdnConnectorRoutetableRouteItem]


class SdnConnectorCompartmentlistItem(TypedDict, total=False):
    """Nested item for compartment-list field."""
    compartment_id: str


class SdnConnectorOciregionlistItem(TypedDict, total=False):
    """Nested item for oci-region-list field."""
    region: str


class SdnConnectorExternalipItem(TypedDict, total=False):
    """Nested item for external-ip field."""
    name: str


class SdnConnectorRouteItem(TypedDict, total=False):
    """Nested item for route field."""
    name: str


class SdnConnectorGcpprojectlistItem(TypedDict, total=False):
    """Nested item for gcp-project-list field."""
    id: str
    gcp_zone_list: str | list[str] | list[SdnConnectorGcpprojectlistGcpzonelistItem]


class SdnConnectorForwardingruleItem(TypedDict, total=False):
    """Nested item for forwarding-rule field."""
    rule_name: str
    target: str


class SdnConnectorPayload(TypedDict, total=False):
    """Payload type for SdnConnector operations."""
    name: str
    status: Literal["disable", "enable"]
    type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"]
    proxy: str
    use_metadata_iam: Literal["disable", "enable"]
    microsoft_365: Literal["disable", "enable"]
    ha_status: Literal["disable", "enable"]
    verify_certificate: Literal["disable", "enable"]
    vdom: str
    server: str
    server_list: str | list[str] | list[SdnConnectorServerlistItem]
    server_port: int
    message_server_port: int
    username: str
    password: str
    vcenter_server: str
    vcenter_username: str
    vcenter_password: str
    access_key: str
    secret_key: str
    region: str
    vpc_id: str
    alt_resource_ip: Literal["disable", "enable"]
    external_account_list: str | list[str] | list[SdnConnectorExternalaccountlistItem]
    tenant_id: str
    client_id: str
    client_secret: str
    subscription_id: str
    resource_group: str
    login_endpoint: str
    resource_url: str
    azure_region: Literal["global", "china", "germany", "usgov", "local"]
    nic: str | list[str] | list[SdnConnectorNicItem]
    route_table: str | list[str] | list[SdnConnectorRoutetableItem]
    user_id: str
    compartment_list: str | list[str] | list[SdnConnectorCompartmentlistItem]
    oci_region_list: str | list[str] | list[SdnConnectorOciregionlistItem]
    oci_region_type: Literal["commercial", "government"]
    oci_cert: str
    oci_fingerprint: str
    external_ip: str | list[str] | list[SdnConnectorExternalipItem]
    route: str | list[str] | list[SdnConnectorRouteItem]
    gcp_project_list: str | list[str] | list[SdnConnectorGcpprojectlistItem]
    forwarding_rule: str | list[str] | list[SdnConnectorForwardingruleItem]
    service_account: str
    private_key: str
    secret_token: str
    domain: str
    group_name: str
    server_cert: str
    server_ca_cert: str
    api_key: str
    ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"]
    par_id: str
    update_interval: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SdnConnectorResponse(TypedDict, total=False):
    """Response type for SdnConnector - use with .dict property for typed dict access."""
    name: str
    status: Literal["disable", "enable"]
    type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"]
    proxy: str
    use_metadata_iam: Literal["disable", "enable"]
    microsoft_365: Literal["disable", "enable"]
    ha_status: Literal["disable", "enable"]
    verify_certificate: Literal["disable", "enable"]
    vdom: str
    server: str
    server_list: list[SdnConnectorServerlistItem]
    server_port: int
    message_server_port: int
    username: str
    password: str
    vcenter_server: str
    vcenter_username: str
    vcenter_password: str
    access_key: str
    secret_key: str
    region: str
    vpc_id: str
    alt_resource_ip: Literal["disable", "enable"]
    external_account_list: list[SdnConnectorExternalaccountlistItem]
    tenant_id: str
    client_id: str
    client_secret: str
    subscription_id: str
    resource_group: str
    login_endpoint: str
    resource_url: str
    azure_region: Literal["global", "china", "germany", "usgov", "local"]
    nic: list[SdnConnectorNicItem]
    route_table: list[SdnConnectorRoutetableItem]
    user_id: str
    compartment_list: list[SdnConnectorCompartmentlistItem]
    oci_region_list: list[SdnConnectorOciregionlistItem]
    oci_region_type: Literal["commercial", "government"]
    oci_cert: str
    oci_fingerprint: str
    external_ip: list[SdnConnectorExternalipItem]
    route: list[SdnConnectorRouteItem]
    gcp_project_list: list[SdnConnectorGcpprojectlistItem]
    forwarding_rule: list[SdnConnectorForwardingruleItem]
    service_account: str
    private_key: str
    secret_token: str
    domain: str
    group_name: str
    server_cert: str
    server_ca_cert: str
    api_key: str
    ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"]
    par_id: str
    update_interval: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SdnConnectorExternalaccountlistRegionlistItemObject(FortiObject[SdnConnectorExternalaccountlistRegionlistItem]):
    """Typed object for external-account-list.region-list table items with attribute access."""
    region: str


class SdnConnectorNicIpItemObject(FortiObject[SdnConnectorNicIpItem]):
    """Typed object for nic.ip table items with attribute access."""
    name: str
    private_ip: str
    public_ip: str
    resource_group: str


class SdnConnectorRoutetableRouteItemObject(FortiObject[SdnConnectorRoutetableRouteItem]):
    """Typed object for route-table.route table items with attribute access."""
    name: str
    next_hop: str


class SdnConnectorGcpprojectlistGcpzonelistItemObject(FortiObject[SdnConnectorGcpprojectlistGcpzonelistItem]):
    """Typed object for gcp-project-list.gcp-zone-list table items with attribute access."""
    name: str


class SdnConnectorServerlistItemObject(FortiObject[SdnConnectorServerlistItem]):
    """Typed object for server-list table items with attribute access."""
    ip: str


class SdnConnectorExternalaccountlistItemObject(FortiObject[SdnConnectorExternalaccountlistItem]):
    """Typed object for external-account-list table items with attribute access."""
    role_arn: str
    external_id: str
    region_list: FortiObjectList[SdnConnectorExternalaccountlistRegionlistItemObject]


class SdnConnectorNicItemObject(FortiObject[SdnConnectorNicItem]):
    """Typed object for nic table items with attribute access."""
    name: str
    peer_nic: str
    ip: FortiObjectList[SdnConnectorNicIpItemObject]


class SdnConnectorRoutetableItemObject(FortiObject[SdnConnectorRoutetableItem]):
    """Typed object for route-table table items with attribute access."""
    name: str
    subscription_id: str
    resource_group: str
    route: FortiObjectList[SdnConnectorRoutetableRouteItemObject]


class SdnConnectorCompartmentlistItemObject(FortiObject[SdnConnectorCompartmentlistItem]):
    """Typed object for compartment-list table items with attribute access."""
    compartment_id: str


class SdnConnectorOciregionlistItemObject(FortiObject[SdnConnectorOciregionlistItem]):
    """Typed object for oci-region-list table items with attribute access."""
    region: str


class SdnConnectorExternalipItemObject(FortiObject[SdnConnectorExternalipItem]):
    """Typed object for external-ip table items with attribute access."""
    name: str


class SdnConnectorRouteItemObject(FortiObject[SdnConnectorRouteItem]):
    """Typed object for route table items with attribute access."""
    name: str


class SdnConnectorGcpprojectlistItemObject(FortiObject[SdnConnectorGcpprojectlistItem]):
    """Typed object for gcp-project-list table items with attribute access."""
    id: str
    gcp_zone_list: FortiObjectList[SdnConnectorGcpprojectlistGcpzonelistItemObject]


class SdnConnectorForwardingruleItemObject(FortiObject[SdnConnectorForwardingruleItem]):
    """Typed object for forwarding-rule table items with attribute access."""
    rule_name: str
    target: str


class SdnConnectorObject(FortiObject):
    """Typed FortiObject for SdnConnector with field access."""
    name: str
    status: Literal["disable", "enable"]
    type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"]
    proxy: str
    use_metadata_iam: Literal["disable", "enable"]
    microsoft_365: Literal["disable", "enable"]
    ha_status: Literal["disable", "enable"]
    verify_certificate: Literal["disable", "enable"]
    server: str
    server_list: FortiObjectList[SdnConnectorServerlistItemObject]
    server_port: int
    message_server_port: int
    username: str
    password: str
    vcenter_server: str
    vcenter_username: str
    vcenter_password: str
    access_key: str
    secret_key: str
    region: str
    vpc_id: str
    alt_resource_ip: Literal["disable", "enable"]
    external_account_list: FortiObjectList[SdnConnectorExternalaccountlistItemObject]
    tenant_id: str
    client_id: str
    client_secret: str
    subscription_id: str
    resource_group: str
    login_endpoint: str
    resource_url: str
    azure_region: Literal["global", "china", "germany", "usgov", "local"]
    nic: FortiObjectList[SdnConnectorNicItemObject]
    route_table: FortiObjectList[SdnConnectorRoutetableItemObject]
    user_id: str
    compartment_list: FortiObjectList[SdnConnectorCompartmentlistItemObject]
    oci_region_list: FortiObjectList[SdnConnectorOciregionlistItemObject]
    oci_region_type: Literal["commercial", "government"]
    oci_cert: str
    oci_fingerprint: str
    external_ip: FortiObjectList[SdnConnectorExternalipItemObject]
    route: FortiObjectList[SdnConnectorRouteItemObject]
    gcp_project_list: FortiObjectList[SdnConnectorGcpprojectlistItemObject]
    forwarding_rule: FortiObjectList[SdnConnectorForwardingruleItemObject]
    service_account: str
    private_key: str
    secret_token: str
    domain: str
    group_name: str
    server_cert: str
    server_ca_cert: str
    api_key: str
    ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"]
    par_id: str
    update_interval: int


# ================================================================
# Main Endpoint Class
# ================================================================

class SdnConnector:
    """
    
    Endpoint: system/sdn_connector
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SdnConnectorObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[SdnConnectorObject]: ...
    
    def get_schema(
        self,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: SdnConnectorPayload | None = ...,
        name: str | None = ...,
        status: Literal["disable", "enable"] | None = ...,
        type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"] | None = ...,
        proxy: str | None = ...,
        use_metadata_iam: Literal["disable", "enable"] | None = ...,
        microsoft_365: Literal["disable", "enable"] | None = ...,
        ha_status: Literal["disable", "enable"] | None = ...,
        verify_certificate: Literal["disable", "enable"] | None = ...,
        server: str | None = ...,
        server_list: str | list[str] | list[SdnConnectorServerlistItem] | None = ...,
        server_port: int | None = ...,
        message_server_port: int | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        vcenter_server: str | None = ...,
        vcenter_username: str | None = ...,
        vcenter_password: str | None = ...,
        access_key: str | None = ...,
        secret_key: str | None = ...,
        region: str | None = ...,
        vpc_id: str | None = ...,
        alt_resource_ip: Literal["disable", "enable"] | None = ...,
        external_account_list: str | list[str] | list[SdnConnectorExternalaccountlistItem] | None = ...,
        tenant_id: str | None = ...,
        client_id: str | None = ...,
        client_secret: str | None = ...,
        subscription_id: str | None = ...,
        resource_group: str | None = ...,
        login_endpoint: str | None = ...,
        resource_url: str | None = ...,
        azure_region: Literal["global", "china", "germany", "usgov", "local"] | None = ...,
        nic: str | list[str] | list[SdnConnectorNicItem] | None = ...,
        route_table: str | list[str] | list[SdnConnectorRoutetableItem] | None = ...,
        user_id: str | None = ...,
        compartment_list: str | list[str] | list[SdnConnectorCompartmentlistItem] | None = ...,
        oci_region_list: str | list[str] | list[SdnConnectorOciregionlistItem] | None = ...,
        oci_region_type: Literal["commercial", "government"] | None = ...,
        oci_cert: str | None = ...,
        oci_fingerprint: str | None = ...,
        external_ip: str | list[str] | list[SdnConnectorExternalipItem] | None = ...,
        route: str | list[str] | list[SdnConnectorRouteItem] | None = ...,
        gcp_project_list: str | list[str] | list[SdnConnectorGcpprojectlistItem] | None = ...,
        forwarding_rule: str | list[str] | list[SdnConnectorForwardingruleItem] | None = ...,
        service_account: str | None = ...,
        private_key: str | None = ...,
        secret_token: str | None = ...,
        domain: str | None = ...,
        group_name: str | None = ...,
        server_cert: str | None = ...,
        server_ca_cert: str | None = ...,
        api_key: str | None = ...,
        ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"] | None = ...,
        par_id: str | None = ...,
        update_interval: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SdnConnectorObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SdnConnectorPayload | None = ...,
        name: str | None = ...,
        status: Literal["disable", "enable"] | None = ...,
        type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"] | None = ...,
        proxy: str | None = ...,
        use_metadata_iam: Literal["disable", "enable"] | None = ...,
        microsoft_365: Literal["disable", "enable"] | None = ...,
        ha_status: Literal["disable", "enable"] | None = ...,
        verify_certificate: Literal["disable", "enable"] | None = ...,
        server: str | None = ...,
        server_list: str | list[str] | list[SdnConnectorServerlistItem] | None = ...,
        server_port: int | None = ...,
        message_server_port: int | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        vcenter_server: str | None = ...,
        vcenter_username: str | None = ...,
        vcenter_password: str | None = ...,
        access_key: str | None = ...,
        secret_key: str | None = ...,
        region: str | None = ...,
        vpc_id: str | None = ...,
        alt_resource_ip: Literal["disable", "enable"] | None = ...,
        external_account_list: str | list[str] | list[SdnConnectorExternalaccountlistItem] | None = ...,
        tenant_id: str | None = ...,
        client_id: str | None = ...,
        client_secret: str | None = ...,
        subscription_id: str | None = ...,
        resource_group: str | None = ...,
        login_endpoint: str | None = ...,
        resource_url: str | None = ...,
        azure_region: Literal["global", "china", "germany", "usgov", "local"] | None = ...,
        nic: str | list[str] | list[SdnConnectorNicItem] | None = ...,
        route_table: str | list[str] | list[SdnConnectorRoutetableItem] | None = ...,
        user_id: str | None = ...,
        compartment_list: str | list[str] | list[SdnConnectorCompartmentlistItem] | None = ...,
        oci_region_list: str | list[str] | list[SdnConnectorOciregionlistItem] | None = ...,
        oci_region_type: Literal["commercial", "government"] | None = ...,
        oci_cert: str | None = ...,
        oci_fingerprint: str | None = ...,
        external_ip: str | list[str] | list[SdnConnectorExternalipItem] | None = ...,
        route: str | list[str] | list[SdnConnectorRouteItem] | None = ...,
        gcp_project_list: str | list[str] | list[SdnConnectorGcpprojectlistItem] | None = ...,
        forwarding_rule: str | list[str] | list[SdnConnectorForwardingruleItem] | None = ...,
        service_account: str | None = ...,
        private_key: str | None = ...,
        secret_token: str | None = ...,
        domain: str | None = ...,
        group_name: str | None = ...,
        server_cert: str | None = ...,
        server_ca_cert: str | None = ...,
        api_key: str | None = ...,
        ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"] | None = ...,
        par_id: str | None = ...,
        update_interval: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SdnConnectorObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SdnConnectorPayload | None = ...,
        name: str | None = ...,
        status: Literal["disable", "enable"] | None = ...,
        type: Literal["aci", "alicloud", "aws", "azure", "gcp", "nsx", "nuage", "oci", "openstack", "kubernetes", "vmware", "sepm", "aci-direct", "ibm", "nutanix", "sap"] | None = ...,
        proxy: str | None = ...,
        use_metadata_iam: Literal["disable", "enable"] | None = ...,
        microsoft_365: Literal["disable", "enable"] | None = ...,
        ha_status: Literal["disable", "enable"] | None = ...,
        verify_certificate: Literal["disable", "enable"] | None = ...,
        server: str | None = ...,
        server_list: str | list[str] | list[SdnConnectorServerlistItem] | None = ...,
        server_port: int | None = ...,
        message_server_port: int | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        vcenter_server: str | None = ...,
        vcenter_username: str | None = ...,
        vcenter_password: str | None = ...,
        access_key: str | None = ...,
        secret_key: str | None = ...,
        region: str | None = ...,
        vpc_id: str | None = ...,
        alt_resource_ip: Literal["disable", "enable"] | None = ...,
        external_account_list: str | list[str] | list[SdnConnectorExternalaccountlistItem] | None = ...,
        tenant_id: str | None = ...,
        client_id: str | None = ...,
        client_secret: str | None = ...,
        subscription_id: str | None = ...,
        resource_group: str | None = ...,
        login_endpoint: str | None = ...,
        resource_url: str | None = ...,
        azure_region: Literal["global", "china", "germany", "usgov", "local"] | None = ...,
        nic: str | list[str] | list[SdnConnectorNicItem] | None = ...,
        route_table: str | list[str] | list[SdnConnectorRoutetableItem] | None = ...,
        user_id: str | None = ...,
        compartment_list: str | list[str] | list[SdnConnectorCompartmentlistItem] | None = ...,
        oci_region_list: str | list[str] | list[SdnConnectorOciregionlistItem] | None = ...,
        oci_region_type: Literal["commercial", "government"] | None = ...,
        oci_cert: str | None = ...,
        oci_fingerprint: str | None = ...,
        external_ip: str | list[str] | list[SdnConnectorExternalipItem] | None = ...,
        route: str | list[str] | list[SdnConnectorRouteItem] | None = ...,
        gcp_project_list: str | list[str] | list[SdnConnectorGcpprojectlistItem] | None = ...,
        forwarding_rule: str | list[str] | list[SdnConnectorForwardingruleItem] | None = ...,
        service_account: str | None = ...,
        private_key: str | None = ...,
        secret_token: str | None = ...,
        domain: str | None = ...,
        group_name: str | None = ...,
        server_cert: str | None = ...,
        server_ca_cert: str | None = ...,
        api_key: str | None = ...,
        ibm_region: Literal["dallas", "washington-dc", "london", "frankfurt", "sydney", "tokyo", "osaka", "toronto", "sao-paulo", "madrid"] | None = ...,
        par_id: str | None = ...,
        update_interval: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "SdnConnector",
    "SdnConnectorPayload",
    "SdnConnectorResponse",
    "SdnConnectorObject",
]


"""
Pydantic Models for CMDB - wireless_controller/wids_profile

Runtime validation models for wireless_controller/wids_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class WidsProfileApScanChannelList6G(BaseModel):
    """
    Child table model for ap-scan-channel-list-6G.
    
    Selected ap scan channel list for 6G band.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    chan: str = Field(max_length=3, description="Channel 6g number.")
class WidsProfileApScanChannelList2G5G(BaseModel):
    """
    Child table model for ap-scan-channel-list-2G-5G.
    
    Selected ap scan channel list for 2.4G and 5G bands.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    chan: str = Field(max_length=3, description="Channel number.")
class WidsProfileApBgscanDisableSchedules(BaseModel):
    """
    Child table model for ap-bgscan-disable-schedules.
    
    Firewall schedules for turning off FortiAP radio background scan. Background scan will be disabled when at least one of the schedules is valid. Separate multiple schedule names with a space.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=35, description="Schedule name.")  # datasource: ['firewall.schedule.group.name', 'firewall.schedule.recurring.name', 'firewall.schedule.onetime.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class WidsProfileModel(BaseModel):
    """
    Pydantic model for wireless_controller/wids_profile configuration.
    
    Configure wireless intrusion detection system (WIDS) profiles.
    
    Validation Rules:        - name: max_length=35 pattern=        - comment: max_length=63 pattern=        - sensor_mode: pattern=        - ap_scan: pattern=        - ap_scan_channel_list_2G_5G: pattern=        - ap_scan_channel_list_6G: pattern=        - ap_bgscan_period: min=10 max=3600 pattern=        - ap_bgscan_intv: min=1 max=600 pattern=        - ap_bgscan_duration: min=10 max=1000 pattern=        - ap_bgscan_idle: min=0 max=1000 pattern=        - ap_bgscan_report_intv: min=15 max=600 pattern=        - ap_bgscan_disable_schedules: pattern=        - ap_fgscan_report_intv: min=15 max=600 pattern=        - ap_scan_passive: pattern=        - ap_scan_threshold: max_length=7 pattern=        - ap_auto_suppress: pattern=        - wireless_bridge: pattern=        - deauth_broadcast: pattern=        - null_ssid_probe_resp: pattern=        - long_duration_attack: pattern=        - long_duration_thresh: min=1000 max=32767 pattern=        - invalid_mac_oui: pattern=        - weak_wep_iv: pattern=        - auth_frame_flood: pattern=        - auth_flood_time: min=5 max=120 pattern=        - auth_flood_thresh: min=1 max=100 pattern=        - assoc_frame_flood: pattern=        - assoc_flood_time: min=5 max=120 pattern=        - assoc_flood_thresh: min=1 max=100 pattern=        - reassoc_flood: pattern=        - reassoc_flood_time: min=1 max=120 pattern=        - reassoc_flood_thresh: min=1 max=65100 pattern=        - probe_flood: pattern=        - probe_flood_time: min=1 max=120 pattern=        - probe_flood_thresh: min=1 max=65100 pattern=        - bcn_flood: pattern=        - bcn_flood_time: min=1 max=120 pattern=        - bcn_flood_thresh: min=1 max=65100 pattern=        - rts_flood: pattern=        - rts_flood_time: min=1 max=120 pattern=        - rts_flood_thresh: min=1 max=65100 pattern=        - cts_flood: pattern=        - cts_flood_time: min=1 max=120 pattern=        - cts_flood_thresh: min=1 max=65100 pattern=        - client_flood: pattern=        - client_flood_time: min=1 max=120 pattern=        - client_flood_thresh: min=1 max=65100 pattern=        - block_ack_flood: pattern=        - block_ack_flood_time: min=1 max=120 pattern=        - block_ack_flood_thresh: min=1 max=65100 pattern=        - pspoll_flood: pattern=        - pspoll_flood_time: min=1 max=120 pattern=        - pspoll_flood_thresh: min=1 max=65100 pattern=        - netstumbler: pattern=        - netstumbler_time: min=1 max=120 pattern=        - netstumbler_thresh: min=1 max=65100 pattern=        - wellenreiter: pattern=        - wellenreiter_time: min=1 max=120 pattern=        - wellenreiter_thresh: min=1 max=65100 pattern=        - spoofed_deauth: pattern=        - asleap_attack: pattern=        - eapol_start_flood: pattern=        - eapol_start_thresh: min=2 max=100 pattern=        - eapol_start_intv: min=1 max=3600 pattern=        - eapol_logoff_flood: pattern=        - eapol_logoff_thresh: min=2 max=100 pattern=        - eapol_logoff_intv: min=1 max=3600 pattern=        - eapol_succ_flood: pattern=        - eapol_succ_thresh: min=2 max=100 pattern=        - eapol_succ_intv: min=1 max=3600 pattern=        - eapol_fail_flood: pattern=        - eapol_fail_thresh: min=2 max=100 pattern=        - eapol_fail_intv: min=1 max=3600 pattern=        - eapol_pre_succ_flood: pattern=        - eapol_pre_succ_thresh: min=2 max=100 pattern=        - eapol_pre_succ_intv: min=1 max=3600 pattern=        - eapol_pre_fail_flood: pattern=        - eapol_pre_fail_thresh: min=2 max=100 pattern=        - eapol_pre_fail_intv: min=1 max=3600 pattern=        - deauth_unknown_src_thresh: min=0 max=65535 pattern=        - windows_bridge: pattern=        - disassoc_broadcast: pattern=        - ap_spoofing: pattern=        - chan_based_mitm: pattern=        - adhoc_valid_ssid: pattern=        - adhoc_network: pattern=        - eapol_key_overflow: pattern=        - ap_impersonation: pattern=        - invalid_addr_combination: pattern=        - beacon_wrong_channel: pattern=        - ht_greenfield: pattern=        - overflow_ie: pattern=        - malformed_ht_ie: pattern=        - malformed_auth: pattern=        - malformed_association: pattern=        - ht_40mhz_intolerance: pattern=        - valid_ssid_misuse: pattern=        - valid_client_misassociation: pattern=        - hotspotter_attack: pattern=        - pwsave_dos_attack: pattern=        - omerta_attack: pattern=        - disconnect_station: pattern=        - unencrypted_valid: pattern=        - fata_jack: pattern=        - risky_encryption: pattern=        - fuzzed_beacon: pattern=        - fuzzed_probe_request: pattern=        - fuzzed_probe_response: pattern=        - air_jack: pattern=        - wpa_ft_attack: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="WIDS profile name.")    
    comment: str | None = Field(max_length=63, default=None, description="Comment.")    
    sensor_mode: Literal["disable", "foreign", "both"] | None = Field(default="disable", description="Scan nearby WiFi stations (default = disable).")    
    ap_scan: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable rogue AP detection.")    
    ap_scan_channel_list_2G_5G: list[WidsProfileApScanChannelList2G5G] = Field(default_factory=list, description="Selected ap scan channel list for 2.4G and 5G bands.")    
    ap_scan_channel_list_6G: list[WidsProfileApScanChannelList6G] = Field(default_factory=list, description="Selected ap scan channel list for 6G band.")    
    ap_bgscan_period: int | None = Field(ge=10, le=3600, default=600, description="Period between background scans (10 - 3600 sec, default = 600).")    
    ap_bgscan_intv: int | None = Field(ge=1, le=600, default=3, description="Period between successive channel scans (1 - 600 sec, default = 3).")    
    ap_bgscan_duration: int | None = Field(ge=10, le=1000, default=30, description="Listen time on scanning a channel (10 - 1000 msec, default = 30).")    
    ap_bgscan_idle: int | None = Field(ge=0, le=1000, default=20, description="Wait time for channel inactivity before scanning this channel (0 - 1000 msec, default = 20).")    
    ap_bgscan_report_intv: int | None = Field(ge=15, le=600, default=30, description="Period between background scan reports (15 - 600 sec, default = 30).")    
    ap_bgscan_disable_schedules: list[WidsProfileApBgscanDisableSchedules] = Field(default_factory=list, description="Firewall schedules for turning off FortiAP radio background scan. Background scan will be disabled when at least one of the schedules is valid. Separate multiple schedule names with a space.")    
    ap_fgscan_report_intv: int | None = Field(ge=15, le=600, default=15, description="Period between foreground scan reports (15 - 600 sec, default = 15).")    
    ap_scan_passive: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable passive scanning. Enable means do not send probe request on any channels (default = disable).")    
    ap_scan_threshold: str | None = Field(max_length=7, default="-90", description="Minimum signal level/threshold in dBm required for the AP to report detected rogue AP (-95 to -20, default = -90).")    
    ap_auto_suppress: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable on-wire rogue AP auto-suppression (default = disable).")    
    wireless_bridge: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable wireless bridge detection (default = disable).")    
    deauth_broadcast: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable broadcasting de-authentication detection (default = disable).")    
    null_ssid_probe_resp: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable null SSID probe response detection (default = disable).")    
    long_duration_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable long duration attack detection based on user configured threshold (default = disable).")    
    long_duration_thresh: int | None = Field(ge=1000, le=32767, default=8200, description="Threshold value for long duration attack detection (1000 - 32767 usec, default = 8200).")    
    invalid_mac_oui: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable invalid MAC OUI detection.")    
    weak_wep_iv: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable weak WEP IV (Initialization Vector) detection (default = disable).")    
    auth_frame_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable authentication frame flooding detection (default = disable).")    
    auth_flood_time: int | None = Field(ge=5, le=120, default=10, description="Number of seconds after which a station is considered not connected.")    
    auth_flood_thresh: int | None = Field(ge=1, le=100, default=30, description="The threshold value for authentication frame flooding.")    
    assoc_frame_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable association frame flooding detection (default = disable).")    
    assoc_flood_time: int | None = Field(ge=5, le=120, default=10, description="Number of seconds after which a station is considered not connected.")    
    assoc_flood_thresh: int | None = Field(ge=1, le=100, default=30, description="The threshold value for association frame flooding.")    
    reassoc_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable reassociation flood detection (default = disable).")    
    reassoc_flood_time: int | None = Field(ge=1, le=120, default=10, description="Detection Window Period.")    
    reassoc_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for reassociation flood.")    
    probe_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable probe flood detection (default = disable).")    
    probe_flood_time: int | None = Field(ge=1, le=120, default=1, description="Detection Window Period.")    
    probe_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for probe flood.")    
    bcn_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable bcn flood detection (default = disable).")    
    bcn_flood_time: int | None = Field(ge=1, le=120, default=1, description="Detection Window Period.")    
    bcn_flood_thresh: int | None = Field(ge=1, le=65100, default=15, description="The threshold value for bcn flood.")    
    rts_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable rts flood detection (default = disable).")    
    rts_flood_time: int | None = Field(ge=1, le=120, default=10, description="Detection Window Period.")    
    rts_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for rts flood.")    
    cts_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable cts flood detection (default = disable).")    
    cts_flood_time: int | None = Field(ge=1, le=120, default=10, description="Detection Window Period.")    
    cts_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for cts flood.")    
    client_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable client flood detection (default = disable).")    
    client_flood_time: int | None = Field(ge=1, le=120, default=10, description="Detection Window Period.")    
    client_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for client flood.")    
    block_ack_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable block_ack flood detection (default = disable).")    
    block_ack_flood_time: int | None = Field(ge=1, le=120, default=1, description="Detection Window Period.")    
    block_ack_flood_thresh: int | None = Field(ge=1, le=65100, default=50, description="The threshold value for block_ack flood.")    
    pspoll_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable pspoll flood detection (default = disable).")    
    pspoll_flood_time: int | None = Field(ge=1, le=120, default=1, description="Detection Window Period.")    
    pspoll_flood_thresh: int | None = Field(ge=1, le=65100, default=30, description="The threshold value for pspoll flood.")    
    netstumbler: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable netstumbler detection (default = disable).")    
    netstumbler_time: int | None = Field(ge=1, le=120, default=30, description="Detection Window Period.")    
    netstumbler_thresh: int | None = Field(ge=1, le=65100, default=5, description="The threshold value for netstumbler.")    
    wellenreiter: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable wellenreiter detection (default = disable).")    
    wellenreiter_time: int | None = Field(ge=1, le=120, default=30, description="Detection Window Period.")    
    wellenreiter_thresh: int | None = Field(ge=1, le=65100, default=5, description="The threshold value for wellenreiter.")    
    spoofed_deauth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable spoofed de-authentication attack detection (default = disable).")    
    asleap_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable asleap attack detection (default = disable).")    
    eapol_start_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable EAPOL-Start flooding (to AP) detection (default = disable).")    
    eapol_start_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for EAPOL-Start flooding in specified interval.")    
    eapol_start_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for EAPOL-Start flooding (1 - 3600 sec).")    
    eapol_logoff_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable EAPOL-Logoff flooding (to AP) detection (default = disable).")    
    eapol_logoff_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for EAPOL-Logoff flooding in specified interval.")    
    eapol_logoff_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for EAPOL-Logoff flooding (1 - 3600 sec).")    
    eapol_succ_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable EAPOL-Success flooding (to AP) detection (default = disable).")    
    eapol_succ_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for EAPOL-Success flooding in specified interval.")    
    eapol_succ_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for EAPOL-Success flooding (1 - 3600 sec).")    
    eapol_fail_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable EAPOL-Failure flooding (to AP) detection (default = disable).")    
    eapol_fail_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for EAPOL-Failure flooding in specified interval.")    
    eapol_fail_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for EAPOL-Failure flooding (1 - 3600 sec).")    
    eapol_pre_succ_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable premature EAPOL-Success flooding (to STA) detection (default = disable).")    
    eapol_pre_succ_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for premature EAPOL-Success flooding in specified interval.")    
    eapol_pre_succ_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for premature EAPOL-Success flooding (1 - 3600 sec).")    
    eapol_pre_fail_flood: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable premature EAPOL-Failure flooding (to STA) detection (default = disable).")    
    eapol_pre_fail_thresh: int | None = Field(ge=2, le=100, default=10, description="The threshold value for premature EAPOL-Failure flooding in specified interval.")    
    eapol_pre_fail_intv: int | None = Field(ge=1, le=3600, default=1, description="The detection interval for premature EAPOL-Failure flooding (1 - 3600 sec).")    
    deauth_unknown_src_thresh: int | None = Field(ge=0, le=65535, default=10, description="Threshold value per second to deauth unknown src for DoS attack (0: no limit).")    
    windows_bridge: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable windows bridge detection (default = disable).")    
    disassoc_broadcast: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable broadcast dis-association detection (default = disable).")    
    ap_spoofing: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP spoofing detection (default = disable).")    
    chan_based_mitm: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable channel based mitm detection (default = disable).")    
    adhoc_valid_ssid: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable adhoc using valid SSID detection (default = disable).")    
    adhoc_network: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable adhoc network detection (default = disable).")    
    eapol_key_overflow: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable overflow EAPOL key detection (default = disable).")    
    ap_impersonation: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP impersonation detection (default = disable).")    
    invalid_addr_combination: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable invalid address combination detection (default = disable).")    
    beacon_wrong_channel: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable beacon wrong channel detection (default = disable).")    
    ht_greenfield: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable HT greenfield detection (default = disable).")    
    overflow_ie: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable overflow IE detection (default = disable).")    
    malformed_ht_ie: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable malformed HT IE detection (default = disable).")    
    malformed_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable malformed auth frame detection (default = disable).")    
    malformed_association: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable malformed association request detection (default = disable).")    
    ht_40mhz_intolerance: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable HT 40 MHz intolerance detection (default = disable).")    
    valid_ssid_misuse: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable valid SSID misuse detection (default = disable).")    
    valid_client_misassociation: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable valid client misassociation detection (default = disable).")    
    hotspotter_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable hotspotter attack detection (default = disable).")    
    pwsave_dos_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable power save DOS attack detection (default = disable).")    
    omerta_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable omerta attack detection (default = disable).")    
    disconnect_station: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable disconnect station detection (default = disable).")    
    unencrypted_valid: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable unencrypted valid detection (default = disable).")    
    fata_jack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable FATA-Jack detection (default = disable).")    
    risky_encryption: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Risky Encryption detection (default = disable).")    
    fuzzed_beacon: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable fuzzed beacon detection (default = disable).")    
    fuzzed_probe_request: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable fuzzed probe request detection (default = disable).")    
    fuzzed_probe_response: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable fuzzed probe response detection (default = disable).")    
    air_jack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AirJack detection (default = disable).")    
    wpa_ft_attack: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WPA FT attack detection (default = disable).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "WidsProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ap_bgscan_disable_schedules_references(self, client: Any) -> list[str]:
        """
        Validate ap_bgscan_disable_schedules references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/schedule/group        - firewall/schedule/recurring        - firewall/schedule/onetime        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WidsProfileModel(
            ...     ap_bgscan_disable_schedules=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ap_bgscan_disable_schedules_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.wids_profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "ap_bgscan_disable_schedules", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.schedule.group.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.recurring.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.onetime.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Ap-Bgscan-Disable-Schedules '{value}' not found in "
                    "firewall/schedule/group or firewall/schedule/recurring or firewall/schedule/onetime"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ap_bgscan_disable_schedules_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "WidsProfileModel",    "WidsProfileApScanChannelList2G5G",    "WidsProfileApScanChannelList6G",    "WidsProfileApBgscanDisableSchedules",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.821860Z
# ============================================================================


"""
Pydantic Models for CMDB - wireless_controller/hotspot20/h2qp_advice_of_charge

Runtime validation models for wireless_controller/hotspot20/h2qp_advice_of_charge configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class H2qpAdviceOfChargeAocListTypeEnum(str, Enum):
    """Allowed values for type_ field in aoc-list."""
    TIME_BASED = "time-based"
    VOLUME_BASED = "volume-based"
    TIME_AND_VOLUME_BASED = "time-and-volume-based"
    UNLIMITED = "unlimited"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class H2qpAdviceOfChargeAocListPlanInfo(BaseModel):
    """
    Child table model for aoc-list.plan-info.
    
    Plan info.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=35, default=None, description="Plan name.")    
    lang: str = Field(max_length=3, description="Language code.")    
    currency: str = Field(max_length=3, description="Currency code.")    
    info_file: str = Field(max_length=255, description="Info file.")
class H2qpAdviceOfChargeAocList(BaseModel):
    """
    Child table model for aoc-list.
    
    AOC list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=35, default=None, description="Advice of charge ID.")    
    type_: H2qpAdviceOfChargeAocListTypeEnum = Field(default=H2qpAdviceOfChargeAocListTypeEnum.TIME_BASED, serialization_alias="type", description="Usage charge type.")    
    nai_realm_encoding: str | None = Field(max_length=1, default=None, description="NAI realm encoding.")    
    nai_realm: str | None = Field(max_length=255, default=None, description="NAI realm list name.")    
    plan_info: list[H2qpAdviceOfChargeAocListPlanInfo] = Field(default_factory=list, description="Plan info.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class H2qpAdviceOfChargeModel(BaseModel):
    """
    Pydantic model for wireless_controller/hotspot20/h2qp_advice_of_charge configuration.
    
    Configure advice of charge.
    
    Validation Rules:        - name: max_length=35 pattern=        - aoc_list: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Plan name.")    
    aoc_list: list[H2qpAdviceOfChargeAocList] = Field(default_factory=list, description="AOC list.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "H2qpAdviceOfChargeModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "H2qpAdviceOfChargeModel",    "H2qpAdviceOfChargeAocList",    "H2qpAdviceOfChargeAocList.PlanInfo",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:02.048688Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/wtp
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class WtpRadio1VapsItem(TypedDict, total=False):
    """Nested item for radio-1.vaps field."""
    name: str


class WtpRadio1ChannelItem(TypedDict, total=False):
    """Nested item for radio-1.channel field."""
    chan: str


class WtpRadio2VapsItem(TypedDict, total=False):
    """Nested item for radio-2.vaps field."""
    name: str


class WtpRadio2ChannelItem(TypedDict, total=False):
    """Nested item for radio-2.channel field."""
    chan: str


class WtpRadio3VapsItem(TypedDict, total=False):
    """Nested item for radio-3.vaps field."""
    name: str


class WtpRadio3ChannelItem(TypedDict, total=False):
    """Nested item for radio-3.channel field."""
    chan: str


class WtpRadio4VapsItem(TypedDict, total=False):
    """Nested item for radio-4.vaps field."""
    name: str


class WtpRadio4ChannelItem(TypedDict, total=False):
    """Nested item for radio-4.channel field."""
    chan: str


class WtpSplittunnelingaclItem(TypedDict, total=False):
    """Nested item for split-tunneling-acl field."""
    id: int
    dest_ip: str


class WtpLanDict(TypedDict, total=False):
    """Nested object type for lan field."""
    port_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port_ssid: str
    port1_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port1_ssid: str
    port2_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port2_ssid: str
    port3_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port3_ssid: str
    port4_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port4_ssid: str
    port5_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port5_ssid: str
    port6_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port6_ssid: str
    port7_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port7_ssid: str
    port8_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port8_ssid: str
    port_esl_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port_esl_ssid: str


class WtpRadio1Dict(TypedDict, total=False):
    """Nested object type for radio-1 field."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str] | list[WtpRadio1VapsItem]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str] | list[WtpRadio1ChannelItem]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio2Dict(TypedDict, total=False):
    """Nested object type for radio-2 field."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str] | list[WtpRadio2VapsItem]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str] | list[WtpRadio2ChannelItem]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio3Dict(TypedDict, total=False):
    """Nested object type for radio-3 field."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str] | list[WtpRadio3VapsItem]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str] | list[WtpRadio3ChannelItem]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio4Dict(TypedDict, total=False):
    """Nested object type for radio-4 field."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str] | list[WtpRadio4VapsItem]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str] | list[WtpRadio4ChannelItem]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpPayload(TypedDict, total=False):
    """Payload type for Wtp operations."""
    wtp_id: str
    index: int
    uuid: str
    admin: Literal["discovered", "disable", "enable"]
    name: str
    location: str
    comment: str
    region: str
    region_x: str
    region_y: str
    firmware_provision: str
    firmware_provision_latest: Literal["disable", "once"]
    wtp_profile: str
    apcfg_profile: str
    bonjour_profile: str
    ble_major_id: int
    ble_minor_id: int
    override_led_state: Literal["enable", "disable"]
    led_state: Literal["enable", "disable"]
    override_wan_port_mode: Literal["enable", "disable"]
    wan_port_mode: Literal["wan-lan", "wan-only"]
    override_ip_fragment: Literal["enable", "disable"]
    ip_fragment_preventing: str | list[str]
    tun_mtu_uplink: int
    tun_mtu_downlink: int
    override_split_tunnel: Literal["enable", "disable"]
    split_tunneling_acl_path: Literal["tunnel", "local"]
    split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"]
    split_tunneling_acl: str | list[str] | list[WtpSplittunnelingaclItem]
    override_lan: Literal["enable", "disable"]
    lan: WtpLanDict
    override_allowaccess: Literal["enable", "disable"]
    allowaccess: str | list[str]
    override_login_passwd_change: Literal["enable", "disable"]
    login_passwd_change: Literal["yes", "default", "no"]
    login_passwd: str
    override_default_mesh_root: Literal["enable", "disable"]
    default_mesh_root: Literal["enable", "disable"]
    radio_1: WtpRadio1Dict
    radio_2: WtpRadio2Dict
    radio_3: WtpRadio3Dict
    radio_4: WtpRadio4Dict
    image_download: Literal["enable", "disable"]
    mesh_bridge_enable: Literal["default", "enable", "disable"]
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    coordinate_latitude: str
    coordinate_longitude: str


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class WtpResponse(TypedDict, total=False):
    """Response type for Wtp - use with .dict property for typed dict access."""
    wtp_id: str
    index: int
    uuid: str
    admin: Literal["discovered", "disable", "enable"]
    name: str
    location: str
    comment: str
    region: str
    region_x: str
    region_y: str
    firmware_provision: str
    firmware_provision_latest: Literal["disable", "once"]
    wtp_profile: str
    apcfg_profile: str
    bonjour_profile: str
    ble_major_id: int
    ble_minor_id: int
    override_led_state: Literal["enable", "disable"]
    led_state: Literal["enable", "disable"]
    override_wan_port_mode: Literal["enable", "disable"]
    wan_port_mode: Literal["wan-lan", "wan-only"]
    override_ip_fragment: Literal["enable", "disable"]
    ip_fragment_preventing: str
    tun_mtu_uplink: int
    tun_mtu_downlink: int
    override_split_tunnel: Literal["enable", "disable"]
    split_tunneling_acl_path: Literal["tunnel", "local"]
    split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"]
    split_tunneling_acl: list[WtpSplittunnelingaclItem]
    override_lan: Literal["enable", "disable"]
    lan: WtpLanDict
    override_allowaccess: Literal["enable", "disable"]
    allowaccess: str
    override_login_passwd_change: Literal["enable", "disable"]
    login_passwd_change: Literal["yes", "default", "no"]
    login_passwd: str
    override_default_mesh_root: Literal["enable", "disable"]
    default_mesh_root: Literal["enable", "disable"]
    radio_1: WtpRadio1Dict
    radio_2: WtpRadio2Dict
    radio_3: WtpRadio3Dict
    radio_4: WtpRadio4Dict
    image_download: Literal["enable", "disable"]
    mesh_bridge_enable: Literal["default", "enable", "disable"]
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    coordinate_latitude: str
    coordinate_longitude: str


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class WtpSplittunnelingaclItemObject(FortiObject[WtpSplittunnelingaclItem]):
    """Typed object for split-tunneling-acl table items with attribute access."""
    id: int
    dest_ip: str


class WtpRadio1VapsItemObject(FortiObject[WtpRadio1VapsItem]):
    """Typed object for radio-1.vaps table items with attribute access."""
    name: str


class WtpRadio1ChannelItemObject(FortiObject[WtpRadio1ChannelItem]):
    """Typed object for radio-1.channel table items with attribute access."""
    chan: str


class WtpRadio2VapsItemObject(FortiObject[WtpRadio2VapsItem]):
    """Typed object for radio-2.vaps table items with attribute access."""
    name: str


class WtpRadio2ChannelItemObject(FortiObject[WtpRadio2ChannelItem]):
    """Typed object for radio-2.channel table items with attribute access."""
    chan: str


class WtpRadio3VapsItemObject(FortiObject[WtpRadio3VapsItem]):
    """Typed object for radio-3.vaps table items with attribute access."""
    name: str


class WtpRadio3ChannelItemObject(FortiObject[WtpRadio3ChannelItem]):
    """Typed object for radio-3.channel table items with attribute access."""
    chan: str


class WtpRadio4VapsItemObject(FortiObject[WtpRadio4VapsItem]):
    """Typed object for radio-4.vaps table items with attribute access."""
    name: str


class WtpRadio4ChannelItemObject(FortiObject[WtpRadio4ChannelItem]):
    """Typed object for radio-4.channel table items with attribute access."""
    chan: str


class WtpLanObject(FortiObject):
    """Nested object for lan field with attribute access."""
    port_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port_ssid: str
    port1_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port1_ssid: str
    port2_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port2_ssid: str
    port3_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port3_ssid: str
    port4_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port4_ssid: str
    port5_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port5_ssid: str
    port6_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port6_ssid: str
    port7_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port7_ssid: str
    port8_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port8_ssid: str
    port_esl_mode: Literal["offline", "nat-to-wan", "bridge-to-wan", "bridge-to-ssid"]
    port_esl_ssid: str


class WtpRadio1Object(FortiObject):
    """Nested object for radio-1 field with attribute access."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio2Object(FortiObject):
    """Nested object for radio-2 field with attribute access."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio3Object(FortiObject):
    """Nested object for radio-3 field with attribute access."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpRadio4Object(FortiObject):
    """Nested object for radio-4 field with attribute access."""
    override_band: Literal["enable", "disable"]
    band: Literal["802.11a", "802.11b", "802.11g", "802.11n-2G", "802.11n-5G", "802.11ac-2G", "802.11ac-5G", "802.11ax-2G", "802.11ax-5G", "802.11ax-6G", "802.11be-2G", "802.11be-5G", "802.11be-6G"]
    override_txpower: Literal["enable", "disable"]
    auto_power_level: Literal["enable", "disable"]
    auto_power_high: int
    auto_power_low: int
    auto_power_target: str
    power_mode: Literal["dBm", "percentage"]
    power_level: int
    power_value: int
    override_vaps: Literal["enable", "disable"]
    vap_all: Literal["tunnel", "bridge", "manual"]
    vaps: str | list[str]
    override_channel: Literal["enable", "disable"]
    channel: str | list[str]
    drma_manual_mode: Literal["ap", "monitor", "ncf", "ncf-peek"]


class WtpObject(FortiObject):
    """Typed FortiObject for Wtp with field access."""
    wtp_id: str
    index: int
    uuid: str
    admin: Literal["discovered", "disable", "enable"]
    name: str
    location: str
    comment: str
    region: str
    region_x: str
    region_y: str
    firmware_provision: str
    firmware_provision_latest: Literal["disable", "once"]
    wtp_profile: str
    apcfg_profile: str
    bonjour_profile: str
    ble_major_id: int
    ble_minor_id: int
    override_led_state: Literal["enable", "disable"]
    led_state: Literal["enable", "disable"]
    override_wan_port_mode: Literal["enable", "disable"]
    wan_port_mode: Literal["wan-lan", "wan-only"]
    override_ip_fragment: Literal["enable", "disable"]
    ip_fragment_preventing: str
    tun_mtu_uplink: int
    tun_mtu_downlink: int
    override_split_tunnel: Literal["enable", "disable"]
    split_tunneling_acl_path: Literal["tunnel", "local"]
    split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"]
    split_tunneling_acl: FortiObjectList[WtpSplittunnelingaclItemObject]
    override_lan: Literal["enable", "disable"]
    lan: WtpLanObject
    override_allowaccess: Literal["enable", "disable"]
    allowaccess: str
    override_login_passwd_change: Literal["enable", "disable"]
    login_passwd_change: Literal["yes", "default", "no"]
    login_passwd: str
    override_default_mesh_root: Literal["enable", "disable"]
    default_mesh_root: Literal["enable", "disable"]
    radio_1: WtpRadio1Object
    radio_2: WtpRadio2Object
    radio_3: WtpRadio3Object
    radio_4: WtpRadio4Object
    image_download: Literal["enable", "disable"]
    mesh_bridge_enable: Literal["default", "enable", "disable"]
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    coordinate_latitude: str
    coordinate_longitude: str


# ================================================================
# Main Endpoint Class
# ================================================================

class Wtp:
    """
    
    Endpoint: wireless_controller/wtp
    Category: cmdb
    MKey: wtp-id
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        wtp_id: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> WtpObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[WtpObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: WtpPayload | None = ...,
        wtp_id: str | None = ...,
        index: int | None = ...,
        uuid: str | None = ...,
        admin: Literal["discovered", "disable", "enable"] | None = ...,
        name: str | None = ...,
        location: str | None = ...,
        comment: str | None = ...,
        region: str | None = ...,
        region_x: str | None = ...,
        region_y: str | None = ...,
        firmware_provision: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        wtp_profile: str | None = ...,
        apcfg_profile: str | None = ...,
        bonjour_profile: str | None = ...,
        ble_major_id: int | None = ...,
        ble_minor_id: int | None = ...,
        override_led_state: Literal["enable", "disable"] | None = ...,
        led_state: Literal["enable", "disable"] | None = ...,
        override_wan_port_mode: Literal["enable", "disable"] | None = ...,
        wan_port_mode: Literal["wan-lan", "wan-only"] | None = ...,
        override_ip_fragment: Literal["enable", "disable"] | None = ...,
        ip_fragment_preventing: str | list[str] | None = ...,
        tun_mtu_uplink: int | None = ...,
        tun_mtu_downlink: int | None = ...,
        override_split_tunnel: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl_path: Literal["tunnel", "local"] | None = ...,
        split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl: str | list[str] | list[WtpSplittunnelingaclItem] | None = ...,
        override_lan: Literal["enable", "disable"] | None = ...,
        lan: WtpLanDict | None = ...,
        override_allowaccess: Literal["enable", "disable"] | None = ...,
        allowaccess: str | list[str] | None = ...,
        override_login_passwd_change: Literal["enable", "disable"] | None = ...,
        login_passwd_change: Literal["yes", "default", "no"] | None = ...,
        login_passwd: str | None = ...,
        override_default_mesh_root: Literal["enable", "disable"] | None = ...,
        default_mesh_root: Literal["enable", "disable"] | None = ...,
        radio_1: WtpRadio1Dict | None = ...,
        radio_2: WtpRadio2Dict | None = ...,
        radio_3: WtpRadio3Dict | None = ...,
        radio_4: WtpRadio4Dict | None = ...,
        image_download: Literal["enable", "disable"] | None = ...,
        mesh_bridge_enable: Literal["default", "enable", "disable"] | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        coordinate_latitude: str | None = ...,
        coordinate_longitude: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> WtpObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: WtpPayload | None = ...,
        wtp_id: str | None = ...,
        index: int | None = ...,
        uuid: str | None = ...,
        admin: Literal["discovered", "disable", "enable"] | None = ...,
        name: str | None = ...,
        location: str | None = ...,
        comment: str | None = ...,
        region: str | None = ...,
        region_x: str | None = ...,
        region_y: str | None = ...,
        firmware_provision: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        wtp_profile: str | None = ...,
        apcfg_profile: str | None = ...,
        bonjour_profile: str | None = ...,
        ble_major_id: int | None = ...,
        ble_minor_id: int | None = ...,
        override_led_state: Literal["enable", "disable"] | None = ...,
        led_state: Literal["enable", "disable"] | None = ...,
        override_wan_port_mode: Literal["enable", "disable"] | None = ...,
        wan_port_mode: Literal["wan-lan", "wan-only"] | None = ...,
        override_ip_fragment: Literal["enable", "disable"] | None = ...,
        ip_fragment_preventing: str | list[str] | None = ...,
        tun_mtu_uplink: int | None = ...,
        tun_mtu_downlink: int | None = ...,
        override_split_tunnel: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl_path: Literal["tunnel", "local"] | None = ...,
        split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl: str | list[str] | list[WtpSplittunnelingaclItem] | None = ...,
        override_lan: Literal["enable", "disable"] | None = ...,
        lan: WtpLanDict | None = ...,
        override_allowaccess: Literal["enable", "disable"] | None = ...,
        allowaccess: str | list[str] | None = ...,
        override_login_passwd_change: Literal["enable", "disable"] | None = ...,
        login_passwd_change: Literal["yes", "default", "no"] | None = ...,
        login_passwd: str | None = ...,
        override_default_mesh_root: Literal["enable", "disable"] | None = ...,
        default_mesh_root: Literal["enable", "disable"] | None = ...,
        radio_1: WtpRadio1Dict | None = ...,
        radio_2: WtpRadio2Dict | None = ...,
        radio_3: WtpRadio3Dict | None = ...,
        radio_4: WtpRadio4Dict | None = ...,
        image_download: Literal["enable", "disable"] | None = ...,
        mesh_bridge_enable: Literal["default", "enable", "disable"] | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        coordinate_latitude: str | None = ...,
        coordinate_longitude: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> WtpObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        wtp_id: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        wtp_id: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: WtpPayload | None = ...,
        wtp_id: str | None = ...,
        index: int | None = ...,
        uuid: str | None = ...,
        admin: Literal["discovered", "disable", "enable"] | None = ...,
        name: str | None = ...,
        location: str | None = ...,
        comment: str | None = ...,
        region: str | None = ...,
        region_x: str | None = ...,
        region_y: str | None = ...,
        firmware_provision: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        wtp_profile: str | None = ...,
        apcfg_profile: str | None = ...,
        bonjour_profile: str | None = ...,
        ble_major_id: int | None = ...,
        ble_minor_id: int | None = ...,
        override_led_state: Literal["enable", "disable"] | None = ...,
        led_state: Literal["enable", "disable"] | None = ...,
        override_wan_port_mode: Literal["enable", "disable"] | None = ...,
        wan_port_mode: Literal["wan-lan", "wan-only"] | None = ...,
        override_ip_fragment: Literal["enable", "disable"] | None = ...,
        ip_fragment_preventing: Literal["tcp-mss-adjust", "icmp-unreachable"] | list[str] | None = ...,
        tun_mtu_uplink: int | None = ...,
        tun_mtu_downlink: int | None = ...,
        override_split_tunnel: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl_path: Literal["tunnel", "local"] | None = ...,
        split_tunneling_acl_local_ap_subnet: Literal["enable", "disable"] | None = ...,
        split_tunneling_acl: str | list[str] | list[WtpSplittunnelingaclItem] | None = ...,
        override_lan: Literal["enable", "disable"] | None = ...,
        lan: WtpLanDict | None = ...,
        override_allowaccess: Literal["enable", "disable"] | None = ...,
        allowaccess: Literal["https", "ssh", "snmp"] | list[str] | None = ...,
        override_login_passwd_change: Literal["enable", "disable"] | None = ...,
        login_passwd_change: Literal["yes", "default", "no"] | None = ...,
        login_passwd: str | None = ...,
        override_default_mesh_root: Literal["enable", "disable"] | None = ...,
        default_mesh_root: Literal["enable", "disable"] | None = ...,
        radio_1: WtpRadio1Dict | None = ...,
        radio_2: WtpRadio2Dict | None = ...,
        radio_3: WtpRadio3Dict | None = ...,
        radio_4: WtpRadio4Dict | None = ...,
        image_download: Literal["enable", "disable"] | None = ...,
        mesh_bridge_enable: Literal["default", "enable", "disable"] | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        coordinate_latitude: str | None = ...,
        coordinate_longitude: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Wtp",
    "WtpPayload",
    "WtpResponse",
    "WtpObject",
]


"""
HFortix FortiOS - Python SDK for FortiGate

Provides comprehensive API client for FortiOS with:
- Full CRUD operations
- Firewall policy management
- Schedule, service, and shaper configuration
- Async support
"""

from __future__ import annotations

import logging
import sys
from typing import Any, Literal

# Import commonly used exceptions
# Import debug utilities from core for convenience
from hfortix_core import (
    APIError,
    AuthenticationError,
    AuthorizationError,
    BadRequestError,
    CircuitBreakerOpenError,
    ConfigurationError,
    DebugSession,
    DuplicateEntryError,
    EntryInUseError,
    FortinetError,
    InvalidValueError,
    MethodNotAllowedError,
    NonRetryableError,
    OperationNotSupportedError,
    PermissionDeniedError,
    RateLimitError,
    ReadOnlyModeError,
    ResourceNotFoundError,
    RetryableError,
    ServerError,
    ServiceUnavailableError,
    TimeoutError,
    VDOMError,
    debug_timer,
    format_connection_stats,
    format_request_info,
    print_debug_info,
)

from .client import FortiOS
from .formatting import to_csv, to_dict, to_json, to_multiline, to_quoted
from .help import help
from .models import FortiObject, FortiObjectList, ContentResponse, CONTENT_ENDPOINTS, is_content_endpoint, parse_fortios_config

# FortiManager proxy support
from .fmg_proxy import (
    FortiManagerProxy,
    ProxyResponse,
    DeviceResult,
)

# Import type definitions for IDE support
from .types import (
    ActionType,
    FortiOSErrorResponse,
    FortiOSResponse,
    FortiOSSuccessResponse,
    FortiOSListResponse,
    FortiOSDictResponse,
    LogSeverity,
    ProtocolType,
    ScheduleType,
    StatusType,
)

__version__ = "0.5.128"

__all__ = [
    # Main client
    "FortiOS",
    "FortiObject",
    "FortiObjectList",
    "ContentResponse",
    "CONTENT_ENDPOINTS",
    "is_content_endpoint",
    "parse_fortios_config",
    "configure_logging",
    # FortiManager proxy
    "FortiManagerProxy",
    "ProxyResponse",
    "DeviceResult",
    # Formatting utilities
    "to_json",
    "to_csv",
    "to_dict",
    "to_multiline",
    "to_quoted",
    # Help system
    "help",
    # Type definitions for IDE support
    "FortiOSSuccessResponse",
    "FortiOSListResponse",
    "FortiOSDictResponse",
    "FortiOSErrorResponse",
    "FortiOSResponse",
    "ActionType",
    "StatusType",
    "LogSeverity",
    "ScheduleType",
    "ProtocolType",
    # Convenience wrappers - commented out, using generated API
    # "FirewallPolicy",
    # "IPMACBindingSetting",
    # "IPMACBindingTable",
    # "ScheduleGroup",
    # "ScheduleOnetime",
    # "ScheduleRecurring",
    # "ServiceCategory",
    # "ServiceCustom",
    # "ServiceGroup",
    # "ShaperPerIp",
    # "TrafficShaper",
    # Debug utilities
    "DebugSession",
    "debug_timer",
    "format_connection_stats",
    "format_request_info",
    "print_debug_info",
    # Exceptions
    "FortinetError",
    "APIError",
    "AuthenticationError",
    "AuthorizationError",
    "RetryableError",
    "NonRetryableError",
    "ConfigurationError",
    "VDOMError",
    "OperationNotSupportedError",
    "ReadOnlyModeError",
    "BadRequestError",
    "ResourceNotFoundError",
    "MethodNotAllowedError",
    "RateLimitError",
    "ServerError",
    "ServiceUnavailableError",
    "CircuitBreakerOpenError",
    "TimeoutError",
    "DuplicateEntryError",
    "EntryInUseError",
    "InvalidValueError",
    "PermissionDeniedError",
]


def configure_logging(
    level: str | int = "INFO",
    format: Literal["json", "text"] = "text",
    handler: logging.Handler | None = None,
    use_color: bool = False,
    include_trace: bool = False,
    output_file: str | None = None,
    structured: bool = False,
) -> None:
    """
    Configure logging for HFortix library

    Provides a convenient way to set up structured logging for all
    HFortix loggers. Useful for enterprise observability with log
    aggregation systems (ELK, Splunk, CloudWatch).

    Args:
        level: Log level (DEBUG, INFO, WARNING, ERROR, CRITICAL) or numeric value  # noqa: E501
        format: Output format - "json" for structured logs or "text" for human-readable  # noqa: E501
        handler: Custom logging handler (optional, default: StreamHandler to stdout)  # noqa: E501
        use_color: Use ANSI color codes in text format (default: False)
        include_trace: Include request/trace IDs in all logs (default: False)
        output_file: Path to log file (logs to both file and console if provided)  # noqa: E501
        structured: Force structured logging with extra fields (default: False)

    Examples:
        Basic text logging:

        >>> from hfortix_fortios import configure_logging, FortiOS
        >>> configure_logging(level="INFO", format="text")
        >>> fgt = FortiOS("192.168.1.99", token="token")

        Structured JSON logging for ELK/Splunk:

        >>> configure_logging(level="INFO", format="json")
        >>> # All logs now output as JSON

        Debug logging with colors:

        >>> configure_logging(level="DEBUG", format="text", use_color=True)
        >>> # Debug logs with ANSI colors

        Log to file:

        >>> configure_logging(level="INFO", format="json", output_file="/var/log/hfortix.log")  # noqa: E501
        >>> # Logs to both console and file

        With request tracing:

        >>> configure_logging(level="INFO", format="json", include_trace=True)
        >>> # All logs include request_id for correlation

    Note:
        This configures all loggers under the "hfortix" namespace:
        - hfortix.http (HTTP client operations)
        - hfortix.audit (Audit logging)
        - hfortix.core (Core utilities)
    """
    # Import formatters from logging package
    from hfortix_core.logging import StructuredFormatter, TextFormatter

    # Get root hfortix logger
    logger = logging.getLogger("hfortix")

    # Convert level string to logging constant
    if isinstance(level, str):
        level = getattr(logging, level.upper(), logging.INFO)

    logger.setLevel(level)

    # Remove existing handlers to avoid duplicates
    logger.handlers.clear()

    # Choose formatter based on format type or structured flag
    if format == "json" or structured:
        formatter: logging.Formatter = StructuredFormatter()
    else:
        formatter = TextFormatter(use_color=use_color)

    # Create console handler if not provided
    if handler is None:
        handler = logging.StreamHandler(sys.stdout)
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    else:
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    # Add file handler if output_file is specified
    if output_file:
        file_handler = logging.FileHandler(output_file)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
        logger.info(f"Logging to file: {output_file}")

    # Prevent propagation to root logger to avoid duplicate logs
    logger.propagate = False

    # Log configuration
    extra: dict[str, Any] = {
        "format": format,
        "level": logging.getLevelName(level),
    }
    if include_trace:
        extra["trace_enabled"] = "true"
    if output_file:
        extra["output_file"] = output_file

    logger.debug(
        f"Logging configured: level={logging.getLevelName(level)}, format={format}",  # noqa: E501
        extra=extra,
    )



"""FortiOS CMDB - Local category"""

from .create import Create
from .import_ import Import

__all__ = [
    "Create",
    "Import",
    "Local",
]


class Local:
    """Local endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Local endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.create = Create(client)
        self.import_ = Import(client)



"""FortiOS CMDB - CentralSnatMap category"""

from ..central_snat_map_base import CentralSnatMap as CentralSnatMapBase
from .clear_counters import ClearCounters
from .reset import Reset

__all__ = [
    "CentralSnatMap",
    "ClearCounters",
    "Reset",
]


class CentralSnatMap(CentralSnatMapBase):
    """CentralSnatMap endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """CentralSnatMap endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.clear_counters = ClearCounters(client)
        self.reset = Reset(client)



"""Validation helpers for extension_controller/extender_vap - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "type",  # Wi-Fi VAP type local-vap / lan-extension-vap.
    "ssid",  # Wi-Fi SSID.
    "passphrase",  # Wi-Fi passphrase.
    "sae-password",  # Wi-Fi SAE Password.
    "auth-server-address",  # Wi-Fi Authentication Server Address (IPv4 format).
    "auth-server-secret",  # Wi-Fi Authentication Server Secret.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "type": "",
    "ssid": "",
    "max-clients": 0,
    "broadcast-ssid": "enable",
    "security": "WPA2-Personal",
    "dtim": 1,
    "rts-threshold": 2347,
    "pmf": "disabled",
    "target-wake-time": "enable",
    "bss-color-partial": "enable",
    "mu-mimo": "enable",
    "auth-server-address": "",
    "auth-server-port": 0,
    "auth-server-secret": "",
    "ip-address": "0.0.0.0 0.0.0.0",
    "start-ip": "0.0.0.0",
    "end-ip": "0.0.0.0",
    "allowaccess": "",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Wi-Fi VAP name.
    "type": "option",  # Wi-Fi VAP type local-vap / lan-extension-vap.
    "ssid": "string",  # Wi-Fi SSID.
    "max-clients": "integer",  # Wi-Fi max clients (0 - 512), default = 0 (no limit) 
    "broadcast-ssid": "option",  # Wi-Fi broadcast SSID enable / disable.
    "security": "option",  # Wi-Fi security.
    "dtim": "integer",  # Wi-Fi DTIM (1 - 255) default = 1.
    "rts-threshold": "integer",  # Wi-Fi RTS Threshold (256 - 2347), default = 2347 (RTS/CTS di
    "pmf": "option",  # Wi-Fi pmf enable/disable, default = disable.
    "target-wake-time": "option",  # Wi-Fi 802.11AX target wake time enable / disable, default = 
    "bss-color-partial": "option",  # Wi-Fi 802.11AX bss color partial enable / disable, default =
    "mu-mimo": "option",  # Wi-Fi multi-user MIMO enable / disable, default = enable.
    "passphrase": "password",  # Wi-Fi passphrase.
    "sae-password": "password",  # Wi-Fi SAE Password.
    "auth-server-address": "string",  # Wi-Fi Authentication Server Address (IPv4 format).
    "auth-server-port": "integer",  # Wi-Fi Authentication Server Port.
    "auth-server-secret": "string",  # Wi-Fi Authentication Server Secret.
    "ip-address": "ipv4-classnet-host",  # Extender ip address.
    "start-ip": "ipv4-address",  # Start ip address.
    "end-ip": "ipv4-address",  # End ip address.
    "allowaccess": "option",  # Control management access to the managed extender. Separate 
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Wi-Fi VAP name.",
    "type": "Wi-Fi VAP type local-vap / lan-extension-vap.",
    "ssid": "Wi-Fi SSID.",
    "max-clients": "Wi-Fi max clients (0 - 512), default = 0 (no limit) ",
    "broadcast-ssid": "Wi-Fi broadcast SSID enable / disable.",
    "security": "Wi-Fi security.",
    "dtim": "Wi-Fi DTIM (1 - 255) default = 1.",
    "rts-threshold": "Wi-Fi RTS Threshold (256 - 2347), default = 2347 (RTS/CTS disabled).",
    "pmf": "Wi-Fi pmf enable/disable, default = disable.",
    "target-wake-time": "Wi-Fi 802.11AX target wake time enable / disable, default = enable.",
    "bss-color-partial": "Wi-Fi 802.11AX bss color partial enable / disable, default = enable.",
    "mu-mimo": "Wi-Fi multi-user MIMO enable / disable, default = enable.",
    "passphrase": "Wi-Fi passphrase.",
    "sae-password": "Wi-Fi SAE Password.",
    "auth-server-address": "Wi-Fi Authentication Server Address (IPv4 format).",
    "auth-server-port": "Wi-Fi Authentication Server Port.",
    "auth-server-secret": "Wi-Fi Authentication Server Secret.",
    "ip-address": "Extender ip address.",
    "start-ip": "Start ip address.",
    "end-ip": "End ip address.",
    "allowaccess": "Control management access to the managed extender. Separate entries with a space.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 15},
    "ssid": {"type": "string", "max_length": 32},
    "max-clients": {"type": "integer", "min": 0, "max": 512},
    "dtim": {"type": "integer", "min": 1, "max": 255},
    "rts-threshold": {"type": "integer", "min": 256, "max": 2347},
    "auth-server-address": {"type": "string", "max_length": 63},
    "auth-server-port": {"type": "integer", "min": 1, "max": 65535},
    "auth-server-secret": {"type": "string", "max_length": 63},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
}


# Valid enum values from API documentation
VALID_BODY_TYPE = [
    "local-vap",
    "lan-ext-vap",
]
VALID_BODY_BROADCAST_SSID = [
    "disable",
    "enable",
]
VALID_BODY_SECURITY = [
    "OPEN",
    "WPA2-Personal",
    "WPA-WPA2-Personal",
    "WPA3-SAE",
    "WPA3-SAE-Transition",
    "WPA2-Enterprise",
    "WPA3-Enterprise-only",
    "WPA3-Enterprise-transition",
    "WPA3-Enterprise-192-bit",
]
VALID_BODY_PMF = [
    "disabled",
    "optional",
    "required",
]
VALID_BODY_TARGET_WAKE_TIME = [
    "disable",
    "enable",
]
VALID_BODY_BSS_COLOR_PARTIAL = [
    "disable",
    "enable",
]
VALID_BODY_MU_MIMO = [
    "disable",
    "enable",
]
VALID_BODY_ALLOWACCESS = [
    "ping",
    "telnet",
    "http",
    "https",
    "ssh",
    "snmp",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_extension_controller_extender_vap_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for extension_controller/extender_vap."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_extension_controller_extender_vap_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new extension_controller/extender_vap object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "broadcast-ssid" in payload:
        is_valid, error = _validate_enum_field(
            "broadcast-ssid",
            payload["broadcast-ssid"],
            VALID_BODY_BROADCAST_SSID,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "security" in payload:
        is_valid, error = _validate_enum_field(
            "security",
            payload["security"],
            VALID_BODY_SECURITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pmf" in payload:
        is_valid, error = _validate_enum_field(
            "pmf",
            payload["pmf"],
            VALID_BODY_PMF,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "target-wake-time" in payload:
        is_valid, error = _validate_enum_field(
            "target-wake-time",
            payload["target-wake-time"],
            VALID_BODY_TARGET_WAKE_TIME,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "bss-color-partial" in payload:
        is_valid, error = _validate_enum_field(
            "bss-color-partial",
            payload["bss-color-partial"],
            VALID_BODY_BSS_COLOR_PARTIAL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "mu-mimo" in payload:
        is_valid, error = _validate_enum_field(
            "mu-mimo",
            payload["mu-mimo"],
            VALID_BODY_MU_MIMO,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "allowaccess" in payload:
        is_valid, error = _validate_enum_field(
            "allowaccess",
            payload["allowaccess"],
            VALID_BODY_ALLOWACCESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_extension_controller_extender_vap_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update extension_controller/extender_vap."""
    # Validate enum values using central function
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "broadcast-ssid" in payload:
        is_valid, error = _validate_enum_field(
            "broadcast-ssid",
            payload["broadcast-ssid"],
            VALID_BODY_BROADCAST_SSID,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "security" in payload:
        is_valid, error = _validate_enum_field(
            "security",
            payload["security"],
            VALID_BODY_SECURITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pmf" in payload:
        is_valid, error = _validate_enum_field(
            "pmf",
            payload["pmf"],
            VALID_BODY_PMF,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "target-wake-time" in payload:
        is_valid, error = _validate_enum_field(
            "target-wake-time",
            payload["target-wake-time"],
            VALID_BODY_TARGET_WAKE_TIME,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "bss-color-partial" in payload:
        is_valid, error = _validate_enum_field(
            "bss-color-partial",
            payload["bss-color-partial"],
            VALID_BODY_BSS_COLOR_PARTIAL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "mu-mimo" in payload:
        is_valid, error = _validate_enum_field(
            "mu-mimo",
            payload["mu-mimo"],
            VALID_BODY_MU_MIMO,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "allowaccess" in payload:
        is_valid, error = _validate_enum_field(
            "allowaccess",
            payload["allowaccess"],
            VALID_BODY_ALLOWACCESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "extension_controller/extender_vap",
    "category": "cmdb",
    "api_path": "extension-controller/extender-vap",
    "mkey": "name",
    "mkey_type": "string",
    "help": "FortiExtender wifi vap configuration.",
    "total_fields": 21,
    "required_fields_count": 6,
    "fields_with_defaults_count": 19,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - system/fortiguard

Runtime validation models for system/fortiguard configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class FortiguardPortEnum(str, Enum):
    """Allowed values for port field."""
    V_8888 = "8888"
    V_53 = "53"
    V_80 = "80"
    V_443 = "443"

class FortiguardAutoFirmwareUpgradeDayEnum(str, Enum):
    """Allowed values for auto_firmware_upgrade_day field."""
    SUNDAY = "sunday"
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"


# ============================================================================
# Main Model
# ============================================================================

class FortiguardModel(BaseModel):
    """
    Pydantic model for system/fortiguard configuration.
    
    Configure FortiGuard services.
    
    Validation Rules:        - fortiguard_anycast: pattern=        - fortiguard_anycast_source: pattern=        - protocol: pattern=        - port: pattern=        - load_balance_servers: min=1 max=266 pattern=        - auto_join_forticloud: pattern=        - update_server_location: pattern=        - sandbox_region: max_length=63 pattern=        - sandbox_inline_scan: pattern=        - update_ffdb: pattern=        - update_uwdb: pattern=        - update_dldb: pattern=        - update_extdb: pattern=        - update_build_proxy: pattern=        - persistent_connection: pattern=        - vdom: max_length=31 pattern=        - auto_firmware_upgrade: pattern=        - auto_firmware_upgrade_day: pattern=        - auto_firmware_upgrade_delay: min=0 max=14 pattern=        - auto_firmware_upgrade_start_hour: min=0 max=23 pattern=        - auto_firmware_upgrade_end_hour: min=0 max=23 pattern=        - FDS_license_expiring_days: min=1 max=100 pattern=        - subscribe_update_notification: pattern=        - antispam_force_off: pattern=        - antispam_cache: pattern=        - antispam_cache_ttl: min=300 max=86400 pattern=        - antispam_cache_mpermille: min=1 max=150 pattern=        - antispam_license: min=0 max=4294967295 pattern=        - antispam_expiration: min=0 max=4294967295 pattern=        - antispam_timeout: min=1 max=30 pattern=        - outbreak_prevention_force_off: pattern=        - outbreak_prevention_cache: pattern=        - outbreak_prevention_cache_ttl: min=300 max=86400 pattern=        - outbreak_prevention_cache_mpermille: min=1 max=150 pattern=        - outbreak_prevention_license: min=0 max=4294967295 pattern=        - outbreak_prevention_expiration: min=0 max=4294967295 pattern=        - outbreak_prevention_timeout: min=1 max=30 pattern=        - webfilter_force_off: pattern=        - webfilter_cache: pattern=        - webfilter_cache_ttl: min=300 max=86400 pattern=        - webfilter_license: min=0 max=4294967295 pattern=        - webfilter_expiration: min=0 max=4294967295 pattern=        - webfilter_timeout: min=1 max=30 pattern=        - sdns_server_ip: pattern=        - sdns_server_port: min=1 max=65535 pattern=        - anycast_sdns_server_ip: pattern=        - anycast_sdns_server_port: min=1 max=65535 pattern=        - sdns_options: pattern=        - source_ip: pattern=        - source_ip6: pattern=        - proxy_server_ip: max_length=63 pattern=        - proxy_server_port: min=0 max=65535 pattern=        - proxy_username: max_length=64 pattern=        - proxy_password: max_length=128 pattern=        - ddns_server_ip: pattern=        - ddns_server_ip6: pattern=        - ddns_server_port: min=1 max=65535 pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    fortiguard_anycast: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable use of FortiGuard's Anycast network.")    
    fortiguard_anycast_source: Literal["fortinet", "aws", "debug"] | None = Field(default="fortinet", description="Configure which of Fortinet's servers to provide FortiGuard services in FortiGuard's anycast network. Default is Fortinet.")    
    protocol: Literal["udp", "http", "https"] | None = Field(default="https", description="Protocol used to communicate with the FortiGuard servers.")    
    port: FortiguardPortEnum | None = Field(default=FortiguardPortEnum.V_443, description="Port used to communicate with the FortiGuard servers.")    
    load_balance_servers: int | None = Field(ge=1, le=266, default=1, description="Number of servers to alternate between as first FortiGuard option.")    
    auto_join_forticloud: Literal["enable", "disable"] | None = Field(default="enable", description="Automatically connect to and login to FortiCloud.")    
    update_server_location: Literal["automatic", "usa", "eu"] | None = Field(default="automatic", description="Location from which to receive FortiGuard updates.")    
    sandbox_region: str | None = Field(max_length=63, default=None, description="FortiCloud Sandbox region.")    
    sandbox_inline_scan: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable FortiCloud Sandbox inline-scan.")    
    update_ffdb: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Internet Service Database update.")    
    update_uwdb: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowlist update.")    
    update_dldb: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable DLP signature update.")    
    update_extdb: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable external resource update.")    
    update_build_proxy: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable proxy dictionary rebuild.")    
    persistent_connection: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of persistent connection to receive update notification from FortiGuard.")    
    vdom: str | None = Field(max_length=31, default=None, description="FortiGuard Service virtual domain name.")  # datasource: ['system.vdom.name']    
    auto_firmware_upgrade: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable automatic patch-level firmware upgrade from FortiGuard. The FortiGate unit searches for new patches only in the same major and minor version.")    
    auto_firmware_upgrade_day: list[FortiguardAutoFirmwareUpgradeDayEnum] = Field(default_factory=list, description="Allowed day(s) of the week to install an automatic patch-level firmware upgrade from FortiGuard (default is none). Disallow any day of the week to use auto-firmware-upgrade-delay instead, which waits for designated days before installing an automatic patch-level firmware upgrade.")    
    auto_firmware_upgrade_delay: int | None = Field(ge=0, le=14, default=3, description="Delay of day(s) before installing an automatic patch-level firmware upgrade from FortiGuard (default = 3). Set it 0 to use auto-firmware-upgrade-day instead, which selects allowed day(s) of the week for installing an automatic patch-level firmware upgrade.")    
    auto_firmware_upgrade_start_hour: int | None = Field(ge=0, le=23, default=1, description="Start time in the designated time window for automatic patch-level firmware upgrade from FortiGuard in 24 hour time (0 ~ 23, default = 2). The actual upgrade time is selected randomly within the time window.")    
    auto_firmware_upgrade_end_hour: int | None = Field(ge=0, le=23, default=4, description="End time in the designated time window for automatic patch-level firmware upgrade from FortiGuard in 24 hour time (0 ~ 23, default = 4). When the end time is smaller than the start time, the end time is interpreted as the next day. The actual upgrade time is selected randomly within the time window.")    
    FDS_license_expiring_days: int | None = Field(ge=1, le=100, default=15, description="Threshold for number of days before FortiGuard license expiration to generate license expiring event log (1 - 100 days, default = 15).")    
    subscribe_update_notification: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable subscription to receive update notification from FortiGuard.")    
    antispam_force_off: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable turning off the FortiGuard antispam service.")    
    antispam_cache: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable FortiGuard antispam request caching. Uses a small amount of memory but improves performance.")    
    antispam_cache_ttl: int | None = Field(ge=300, le=86400, default=1800, description="Time-to-live for antispam cache entries in seconds (300 - 86400). Lower times reduce the cache size. Higher times may improve performance since the cache will have more entries.")    
    antispam_cache_mpermille: int | None = Field(ge=1, le=150, default=1, description="Maximum permille of FortiGate memory the antispam cache is allowed to use (1 - 150).")    
    antispam_license: int | None = Field(ge=0, le=4294967295, default=4294967295, description="Interval of time between license checks for the FortiGuard antispam contract.")    
    antispam_expiration: int | None = Field(ge=0, le=4294967295, default=0, description="Expiration date of the FortiGuard antispam contract.")    
    antispam_timeout: int = Field(ge=1, le=30, default=7, description="Antispam query time out (1 - 30 sec, default = 7).")    
    outbreak_prevention_force_off: Literal["enable", "disable"] | None = Field(default="disable", description="Turn off FortiGuard Virus Outbreak Prevention service.")    
    outbreak_prevention_cache: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable FortiGuard Virus Outbreak Prevention cache.")    
    outbreak_prevention_cache_ttl: int | None = Field(ge=300, le=86400, default=300, description="Time-to-live for FortiGuard Virus Outbreak Prevention cache entries (300 - 86400 sec, default = 300).")    
    outbreak_prevention_cache_mpermille: int | None = Field(ge=1, le=150, default=1, description="Maximum permille of memory FortiGuard Virus Outbreak Prevention cache can use (1 - 150 permille, default = 1).")    
    outbreak_prevention_license: int | None = Field(ge=0, le=4294967295, default=4294967295, description="Interval of time between license checks for FortiGuard Virus Outbreak Prevention contract.")    
    outbreak_prevention_expiration: int | None = Field(ge=0, le=4294967295, default=0, description="Expiration date of FortiGuard Virus Outbreak Prevention contract.")    
    outbreak_prevention_timeout: int = Field(ge=1, le=30, default=7, description="FortiGuard Virus Outbreak Prevention time out (1 - 30 sec, default = 7).")    
    webfilter_force_off: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable turning off the FortiGuard web filtering service.")    
    webfilter_cache: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable FortiGuard web filter caching.")    
    webfilter_cache_ttl: int | None = Field(ge=300, le=86400, default=3600, description="Time-to-live for web filter cache entries in seconds (300 - 86400).")    
    webfilter_license: int | None = Field(ge=0, le=4294967295, default=4294967295, description="Interval of time between license checks for the FortiGuard web filter contract.")    
    webfilter_expiration: int | None = Field(ge=0, le=4294967295, default=0, description="Expiration date of the FortiGuard web filter contract.")    
    webfilter_timeout: int = Field(ge=1, le=30, default=15, description="Web filter query time out (1 - 30 sec, default = 15).")    
    sdns_server_ip: list[str] = Field(default_factory=list, description="IP address of the FortiGuard DNS rating server.")    
    sdns_server_port: int | None = Field(ge=1, le=65535, default=53, description="Port to connect to on the FortiGuard DNS rating server.")    
    anycast_sdns_server_ip: str | None = Field(default="0.0.0.0", description="IP address of the FortiGuard anycast DNS rating server.")    
    anycast_sdns_server_port: int | None = Field(ge=1, le=65535, default=853, description="Port to connect to on the FortiGuard anycast DNS rating server.")    
    sdns_options: list[Literal["include-question-section"]] = Field(default_factory=list, description="Customization options for the FortiGuard DNS service.")    
    source_ip: str | None = Field(default="0.0.0.0", description="Source IPv4 address used to communicate with FortiGuard.")    
    source_ip6: str | None = Field(default="::", description="Source IPv6 address used to communicate with FortiGuard.")    
    proxy_server_ip: str | None = Field(max_length=63, default=None, description="Hostname or IPv4 address of the proxy server.")    
    proxy_server_port: int | None = Field(ge=0, le=65535, default=0, description="Port used to communicate with the proxy server.")    
    proxy_username: str | None = Field(max_length=64, default=None, description="Proxy user name.")    
    proxy_password: Any = Field(max_length=128, default=None, description="Proxy user password.")    
    ddns_server_ip: str | None = Field(default="0.0.0.0", description="IP address of the FortiDDNS server.")    
    ddns_server_ip6: str | None = Field(default="::", description="IPv6 address of the FortiDDNS server.")    
    ddns_server_port: int | None = Field(ge=1, le=65535, default=443, description="Port used to communicate with FortiDDNS servers.")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('vdom')
    @classmethod
    def validate_vdom(cls, v: Any) -> Any:
        """
        Validate vdom field.
        
        Datasource: ['system.vdom.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "FortiguardModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_vdom_references(self, client: Any) -> list[str]:
        """
        Validate vdom references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/vdom        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FortiguardModel(
            ...     vdom="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vdom_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.fortiguard.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vdom", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.vdom.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vdom '{value}' not found in "
                "system/vdom"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FortiguardModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.fortiguard.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_vdom_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "FortiguardModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.801749Z
# ============================================================================


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_SENSOR_MODE: Literal["disable", "foreign", "both"]
VALID_BODY_AP_SCAN: Literal["disable", "enable"]
VALID_BODY_AP_SCAN_PASSIVE: Literal["enable", "disable"]
VALID_BODY_AP_AUTO_SUPPRESS: Literal["enable", "disable"]
VALID_BODY_WIRELESS_BRIDGE: Literal["enable", "disable"]
VALID_BODY_DEAUTH_BROADCAST: Literal["enable", "disable"]
VALID_BODY_NULL_SSID_PROBE_RESP: Literal["enable", "disable"]
VALID_BODY_LONG_DURATION_ATTACK: Literal["enable", "disable"]
VALID_BODY_INVALID_MAC_OUI: Literal["enable", "disable"]
VALID_BODY_WEAK_WEP_IV: Literal["enable", "disable"]
VALID_BODY_AUTH_FRAME_FLOOD: Literal["enable", "disable"]
VALID_BODY_ASSOC_FRAME_FLOOD: Literal["enable", "disable"]
VALID_BODY_REASSOC_FLOOD: Literal["enable", "disable"]
VALID_BODY_PROBE_FLOOD: Literal["enable", "disable"]
VALID_BODY_BCN_FLOOD: Literal["enable", "disable"]
VALID_BODY_RTS_FLOOD: Literal["enable", "disable"]
VALID_BODY_CTS_FLOOD: Literal["enable", "disable"]
VALID_BODY_CLIENT_FLOOD: Literal["enable", "disable"]
VALID_BODY_BLOCK_ACK_FLOOD: Literal["enable", "disable"]
VALID_BODY_PSPOLL_FLOOD: Literal["enable", "disable"]
VALID_BODY_NETSTUMBLER: Literal["enable", "disable"]
VALID_BODY_WELLENREITER: Literal["enable", "disable"]
VALID_BODY_SPOOFED_DEAUTH: Literal["enable", "disable"]
VALID_BODY_ASLEAP_ATTACK: Literal["enable", "disable"]
VALID_BODY_EAPOL_START_FLOOD: Literal["enable", "disable"]
VALID_BODY_EAPOL_LOGOFF_FLOOD: Literal["enable", "disable"]
VALID_BODY_EAPOL_SUCC_FLOOD: Literal["enable", "disable"]
VALID_BODY_EAPOL_FAIL_FLOOD: Literal["enable", "disable"]
VALID_BODY_EAPOL_PRE_SUCC_FLOOD: Literal["enable", "disable"]
VALID_BODY_EAPOL_PRE_FAIL_FLOOD: Literal["enable", "disable"]
VALID_BODY_WINDOWS_BRIDGE: Literal["enable", "disable"]
VALID_BODY_DISASSOC_BROADCAST: Literal["enable", "disable"]
VALID_BODY_AP_SPOOFING: Literal["enable", "disable"]
VALID_BODY_CHAN_BASED_MITM: Literal["enable", "disable"]
VALID_BODY_ADHOC_VALID_SSID: Literal["enable", "disable"]
VALID_BODY_ADHOC_NETWORK: Literal["enable", "disable"]
VALID_BODY_EAPOL_KEY_OVERFLOW: Literal["enable", "disable"]
VALID_BODY_AP_IMPERSONATION: Literal["enable", "disable"]
VALID_BODY_INVALID_ADDR_COMBINATION: Literal["enable", "disable"]
VALID_BODY_BEACON_WRONG_CHANNEL: Literal["enable", "disable"]
VALID_BODY_HT_GREENFIELD: Literal["enable", "disable"]
VALID_BODY_OVERFLOW_IE: Literal["enable", "disable"]
VALID_BODY_MALFORMED_HT_IE: Literal["enable", "disable"]
VALID_BODY_MALFORMED_AUTH: Literal["enable", "disable"]
VALID_BODY_MALFORMED_ASSOCIATION: Literal["enable", "disable"]
VALID_BODY_HT_40MHZ_INTOLERANCE: Literal["enable", "disable"]
VALID_BODY_VALID_SSID_MISUSE: Literal["enable", "disable"]
VALID_BODY_VALID_CLIENT_MISASSOCIATION: Literal["enable", "disable"]
VALID_BODY_HOTSPOTTER_ATTACK: Literal["enable", "disable"]
VALID_BODY_PWSAVE_DOS_ATTACK: Literal["enable", "disable"]
VALID_BODY_OMERTA_ATTACK: Literal["enable", "disable"]
VALID_BODY_DISCONNECT_STATION: Literal["enable", "disable"]
VALID_BODY_UNENCRYPTED_VALID: Literal["enable", "disable"]
VALID_BODY_FATA_JACK: Literal["enable", "disable"]
VALID_BODY_RISKY_ENCRYPTION: Literal["enable", "disable"]
VALID_BODY_FUZZED_BEACON: Literal["enable", "disable"]
VALID_BODY_FUZZED_PROBE_REQUEST: Literal["enable", "disable"]
VALID_BODY_FUZZED_PROBE_RESPONSE: Literal["enable", "disable"]
VALID_BODY_AIR_JACK: Literal["enable", "disable"]
VALID_BODY_WPA_FT_ATTACK: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_SENSOR_MODE",
    "VALID_BODY_AP_SCAN",
    "VALID_BODY_AP_SCAN_PASSIVE",
    "VALID_BODY_AP_AUTO_SUPPRESS",
    "VALID_BODY_WIRELESS_BRIDGE",
    "VALID_BODY_DEAUTH_BROADCAST",
    "VALID_BODY_NULL_SSID_PROBE_RESP",
    "VALID_BODY_LONG_DURATION_ATTACK",
    "VALID_BODY_INVALID_MAC_OUI",
    "VALID_BODY_WEAK_WEP_IV",
    "VALID_BODY_AUTH_FRAME_FLOOD",
    "VALID_BODY_ASSOC_FRAME_FLOOD",
    "VALID_BODY_REASSOC_FLOOD",
    "VALID_BODY_PROBE_FLOOD",
    "VALID_BODY_BCN_FLOOD",
    "VALID_BODY_RTS_FLOOD",
    "VALID_BODY_CTS_FLOOD",
    "VALID_BODY_CLIENT_FLOOD",
    "VALID_BODY_BLOCK_ACK_FLOOD",
    "VALID_BODY_PSPOLL_FLOOD",
    "VALID_BODY_NETSTUMBLER",
    "VALID_BODY_WELLENREITER",
    "VALID_BODY_SPOOFED_DEAUTH",
    "VALID_BODY_ASLEAP_ATTACK",
    "VALID_BODY_EAPOL_START_FLOOD",
    "VALID_BODY_EAPOL_LOGOFF_FLOOD",
    "VALID_BODY_EAPOL_SUCC_FLOOD",
    "VALID_BODY_EAPOL_FAIL_FLOOD",
    "VALID_BODY_EAPOL_PRE_SUCC_FLOOD",
    "VALID_BODY_EAPOL_PRE_FAIL_FLOOD",
    "VALID_BODY_WINDOWS_BRIDGE",
    "VALID_BODY_DISASSOC_BROADCAST",
    "VALID_BODY_AP_SPOOFING",
    "VALID_BODY_CHAN_BASED_MITM",
    "VALID_BODY_ADHOC_VALID_SSID",
    "VALID_BODY_ADHOC_NETWORK",
    "VALID_BODY_EAPOL_KEY_OVERFLOW",
    "VALID_BODY_AP_IMPERSONATION",
    "VALID_BODY_INVALID_ADDR_COMBINATION",
    "VALID_BODY_BEACON_WRONG_CHANNEL",
    "VALID_BODY_HT_GREENFIELD",
    "VALID_BODY_OVERFLOW_IE",
    "VALID_BODY_MALFORMED_HT_IE",
    "VALID_BODY_MALFORMED_AUTH",
    "VALID_BODY_MALFORMED_ASSOCIATION",
    "VALID_BODY_HT_40MHZ_INTOLERANCE",
    "VALID_BODY_VALID_SSID_MISUSE",
    "VALID_BODY_VALID_CLIENT_MISASSOCIATION",
    "VALID_BODY_HOTSPOTTER_ATTACK",
    "VALID_BODY_PWSAVE_DOS_ATTACK",
    "VALID_BODY_OMERTA_ATTACK",
    "VALID_BODY_DISCONNECT_STATION",
    "VALID_BODY_UNENCRYPTED_VALID",
    "VALID_BODY_FATA_JACK",
    "VALID_BODY_RISKY_ENCRYPTION",
    "VALID_BODY_FUZZED_BEACON",
    "VALID_BODY_FUZZED_PROBE_REQUEST",
    "VALID_BODY_FUZZED_PROBE_RESPONSE",
    "VALID_BODY_AIR_JACK",
    "VALID_BODY_WPA_FT_ATTACK",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]